#!/usr/bin/env ruby

# ----------------------------------------------------------------------------------------------------
#
# Based on the export script by wkoffel [http://forums.omnigroup.com/showpost.php?p=22739&postcount=1]
#
# Version 0.2.4
# 
# Modified to export export the tasks into a Safari bookmark and 
# therefore be syncable to the iPhone and available for offline viewing
#
# Before running the script, create a new bookmark with the address "data:text/html"
# (without the quotes). The name can be whatever you like, my name is "ToDo"
#
# 12/28/07 Modified to add hyperlinks for notes
# ----------------------------------------------------------------------------------------------------

%w[rubygems markaby appscript base64].each { |l| require l }
include Appscript

def linkify_text(string)
  txt = string.dup
  txt.gsub!(/http(s)?:\/\/([\w-]+\.)+[\w-]+(\/[ .\/?%&=\w-]*)*/, '<a href="\&">\&</a>')
  txt
end

def markup_for_task(task)
  project_name     = task.containing_project.name.get
  parent_task_name = task.parent_task.name.get rescue ''
  task_note        = linkify_text(task.note.get)
  
  p { "Project: #{project_name}" }
  p { "Subtask of: #{parent_task_name}" } if parent_task_name != project_name
  p { "Notes: #{task_note}" } unless(task_note == "")
end

def each_context(c, parent = '')
  parent = "#{parent} : " unless parent == ''
  
  if c.available_tasks.get.any?
    li.group parent + c.name.get.to_s
    c.available_tasks.get.each do |task|
      li.task :style => 'display: none;' do |l|
        l << span.project_name { task.containing_project.name.get }
        l << task.name.get
        div.description :style => 'display: none;' do
          markup_for_task(task)
        end
      end
    end
  end
    
  c.contexts.get.each do |sub_context|
    each_context(sub_context, parent + c.name.get.to_s)
  end  
end

class Plist
  def self.edit_as_xml(filename, &block)
    return unless block_given?
    convert_plist(filename, :to => :xml)
    new_contents = yield File.read(filename)
    File.open(filename, 'w').write(new_contents)
    self.convert_plist(filename, :to => :binary)
  end

  def self.convert_plist(filename, options = {})
    options[:to] ||= :xml
    `plutil -convert #{options[:to].to_s}1 #{filename}`
  end
end

# ------------------------------------------------------------------------------------

mab = Markaby::Builder.new(:indent => 1, :auto_validation => false)
dir = File.dirname(__FILE__)

mab.html do
  head do 
    title "OmniFocus Tasks"
    meta :name => "viewport", :content => "width=320; initial-scale=1.0; maximum-scale=1.0; user-scalable=0;"
    style :type => "text/css" do |css|
      css << File.read(dir + '/Resources/iui.css')
      css << File.read(dir + '/Resources/style.css')
    end
    script :type => "text/javascript" do
      File.read(dir + '/Resources/script.js')
    end
  end
  
  body do
    div.toolbar do
      h1.pageTitle! "OmniFocus"
    end
    
    context = app('OmniFocus').documents[1]
    tag! :ul, { :selected => true, :title => context.name.get } do
      context.contexts.get.each { |c| each_context(c) } if context.contexts.get.any?
    end
  end
end

# ------------------------------------------------------------------------------------

BOOKMARK_FILE  = '~/Library/Safari/Bookmarks.plist'
NOTIFY_TITLE   = 'OmniFocus Tasks Exported'
NOTIFY_MESSAGE = 'All active OmniFocus tasks was exported to Safari as a bookmark.'

# Base64 encode our data to use in the bookmark
data = Base64.encode64(mab.to_s + "\n").gsub(/\s+/, '')
new_todo = "DATA:text/html;charset=utf-8;base64,#{data}"

# Convert Safari's bookmark plist file to text/xml
bookmark_file = File.expand_path(BOOKMARK_FILE)

# Convert bookmarks.plist to (editable) xml
Plist.edit_as_xml(bookmark_file) do |contents|
  begin

    # TODO: Clean up this ugly mess!
    # There are two occurrences of each url, to be safe we replace both instances
    2.times do
      start                  = contents.index('<string>data:text/html')
      stop                   = contents.index('</string>', start)
      current_todo           = contents[start + '<string>'.length, stop - start - '</string>'.length + 1]
      contents[current_todo] = new_todo
    end
    
    contents.gsub!('DATA:text/html', 'data:text/html') # Hackity hack :)
    contents
    
  rescue TypeError => e
    puts 'Cannot find any bookmark with address "data:text/html" to replace.'
    exit
  end 
end

`/usr/local/bin/growlnotify  --iconpath "#{File.dirname(__FILE__)}/Sync OmniFocus with Safari.app" -t '#{NOTIFY_TITLE}' -m '#{NOTIFY_MESSAGE}' &> /dev/null`
